/*************************************************
 Alarm z powiadomieniem przez SMS oraz wyczany 
 przez odbir identyfikacji osoby dzwonicej CLIP
 
 UWAGA: przystosowany do pracy z telefonami
 GSM firmy SIEMENS M/S/C35
 *************************************************/

#pragma DEFJ(TIM1_INIT=0xFD)		//Timer 1 jako zegar UART (19200,n,8,1)
											//warto TH1 = 0xFD dla 11,0592MHz 
#pragma SMALL							//wybr modelu pamici

#include <stdio.h>					//funkcje wejcia - wyjcia
#include <string.h>					//operacje na acuchach
#include <siemens35.h>				//biblioteka obsugi SIEMENS35
#include <reg8252.h>					//definicje rejestrw AT89S8252
#include <eeprom8252.h>				//biblioteka funkcji obsugi pamici eeprom AT89S8252

//definicje staych
#define OFF 		0
#define ON			1
#define ACTIVE		0x0F
#define DISACTIVE	0xF0
#define time2on	8					//czas opnienia do zaczenia alarmu
#define alarmtime	60					//przybliony czas zaczenia alarmu w sekundach

//definicja zewntrzna komunikatu wysyanego przez program
extern code char TEKSTPOWIADOMIENIA[];
//definicja zewntrzna "telefonu supervisora"
extern code char SPHONE[];

//bity zaczajce ukady wykonawcze
sbit RELAY_1 = P3^5;
sbit RELAY_2 = P3^4;
sbit BUZZER = P1^4;
sbit NIBBLE = P3^2;

//zmienna okrelajca tryb pracy alarmu
data BYTE INWATCHMODE;
//zmienna zawierajca stan czujnikw
data BYTE SENSORS;
//licznik czasu opnienia
data BYTE DELAYTIME;
//bit znacznika ustawiany przez uprawniony do wyczenia alarmu telefon; naley zaczy identyfikacj
//osoby dzwonicej-jeli alarm odbierze ten telefon, to zostanie wyczony;bit ustawiany w obsudze przerwania
bit NOPHONE = 1;
//bufor odbioru dla UART
at 0x0400 xdata BYTE RBUF[256];
//indeks bufora odbioru
data BYTE RECVD = 0;
//licznik odebranych znakw nowej linii (0x0A)
data BYTE OxOA_CNT = 0;
//numer odczytany z pamici telefonu
data BYTE READPHONE[13];


//procedura obsugi przerwania UART
void UART_Isr() interrupt 4 using 2
{
	BYTE temp;
	
	if (RI != 0)									//fragment kodu wykonywany, gdy ustawiona jest
	{													//flaga odbioru
		RI = 0;
		temp = SBUF;
		if (temp > 0x0D) RBUF[RECVD++] = temp;
		if (temp == 0x0A) OxOA_CNT++;
	}
}


//sygnalizacja bdu przy pomocy LED: 2 krtkie byski+0,5s przerwy
void comunicationerror()
{
	while(1)
	{
		SCK = 0; delay(80);
		SCK = 1; delay(150);
		SCK = 0; delay(80);
		SCK = 1; delay(500);
	}
}


//sprawdzenie numeru dzwonicego telefonu, dane pobierane z bufora UART
bit checknumber()
{
	data BYTE T[13];
	signed char compresult;
	
	EA = 0;											//blokowanie przerwa
//-----------------------------------------------------------------------------------------------------------------
	strncpy(&SPHONE, &READPHONE, 10);		//kopiowanie numeru uprawnionego z ROM do zmiennej w RAM;
														//UWAGA:moe by zastpione przez odczyt numeru zapamitanego na przykad
														//na pozycji 1 SMS w pamici telefonu czy karty SIM!
//-----------------------------------------------------------------------------------------------------------------
	RECVD = 0;										//wyszukanie "G" i ustawienie indeksu na "G" + 8 (CR-LF zostay pominite)
	while ((RBUF[RECVD] != 'G') && (RECVD < 256)) RECVD++;
	if (RECVD < 256)								//jeli brak znaku "G" - wyjcie z funkcji
	{
		disconnect();								//poczenie sprawdzone,rozczenie
		strncpy(&T, &RBUF, 10);					//uformowanie acucha do porwna
		compresult = strcmp(&T, &READPHONE);//porwnanie tekstw numeru z ROM z numerem telefonu dzwonicego
														//funkcja zwraca warto "0", jeli numery s zgodne
	}
	RECVD = OxOA_CNT = 0;						//zerowanie licznika i indeksu
	EA = 1;											//zaczenie przerwa
	return(compresult == 0);
}


//obsuga zaczenia alarmu
void alarmon()
{
	BYTE i;
	
	DELAYTIME = alarmtime;						//licznik czasu zaczenia alarmu
	RELAY_1= ON; 									//zaczenie przekanika RL1
	SCK = 0;											//zaczenie diody LED
	NOPHONE = 1;
	sendsms(TEKSTPOWIADOMIENIA);
	while ((DELAYTIME--)&& NOPHONE)			//w tej ptli program oczekuje na wyczenie w postaci przekroczenia czasu
	{													//przewidzianego dla alarmu lub wyczenie na skutek odebrania "uprawnionego" 
														//telefonu (numer jest wpisany do pamici telefonu - modemu GSM)
		RELAY_2 = ON;								//ten przekanik bdzie pracowa "pulsacyjnie"
		delay(2000);								//bdzie zaczany na 2s. w odstpie 1s.
		RELAY_2 = OFF;
		delay(1000);
		if (OxOA_CNT == 4) NOPHONE = checknumber();
	}
	if (NOPHONE == 0) 							//jeli wyczono alarm "normalnie",wyczenie trybu czuwania
	{													//oraz wyczenie przerwa
		INWATCHMODE = DISACTIVE;
		EA = 0;
	}
}


//odczyt wej czujnikw alarmowych (stan zwracany w SENSORS,aktywna 1)
void readsensors()
{
	BYTE r1;
	
	NIBBLE = 0;										//odczyt powki umownie uznanej za starsz
	SENSORS = (P1 << 4) & 0xF0;
	NIBBLE = 1;										//odczyt powki umownie uznanej za modsz
	r1 = P1;
	r1 &= 0x0F;
	SENSORS |= r1;									//sumowanie obu powek
	SENSORS = ~SENSORS;							//teraz "1" jest stanem aktywnym, podczas gdy zadziaanie czujki wymusza "0"
}


//program gwny
void main()
{
	WMCON |= 0x02;									//zerowanie timer'a watchdog
	WMCON &= 0xFE;									//zatrzymanie watchdog'a, jeli uK zosta zrestartowany
	SCK = 0;											//zaczenie LED
	delay(1000);									//czas na stabilizacj warunkw pracy
	PCON |= 0x80;									//ustawienie bitu prescalera zegara transmisji

	RELAY_1 = RELAY_2 = BUZZER = OFF;		//warunki pocztkowe pracy przekanikw
	REN = ES =1;									//zezwolenie na prac odbiornika UART i na przyjmowanie przerwa od UART
	
	INWATCHMODE = ee_getbyte(0); 				//status alarmu pobierany z eeprom,np.po restarcie watchdog,wyczeniu
														//zasilania i temu podobnych sytuacjach
	if (INWATCHMODE == 0xFF)					//inicjalizacja,jeli pami eeprom nie bya uywana (pierwsze zaczenie)
	{
		ee_savebyte(DISACTIVE, 0);
		INWATCHMODE = DISACTIVE;
	}

	if (!initgsm())								//funkcja zwraca warto TRUE, jeli inicjalizacja telefonu jest OK
														//jeli bd komunikacji z telefonem,to jest on sygnalizowany przez
		comunicationerror();						//szybkie mruganie LED

//------------------ GWNA PTLA PROGRAMU ------------------
	while (1) 
	{
loopbegin:
		if (INWATCHMODE == ACTIVE)
		{
			EA = 1;									//zaczenie przerwa dla odbioru ew.sygnau dzwonienia
			if (OxOA_CNT == 4) NOPHONE = checknumber();	//odebrano identyfikacj CLIP
			if (NOPHONE == 0)						//jeli funkcja sprawdzia,e to uprawniony telefon,to wyczenie alarmu
			{
				INWATCHMODE = DISACTIVE;		//wyczenie trybu czuwania
				EA = 0;								//wyczenie przerwa
				SCK = 1;								//wyczenie diody LED
				goto loopbegin;					//powrt na pocztek ptli
			}
			SCK = 0;									//zaczenie diody LED
			readsensors();							//odczyt czujnikw,warto odczytu w zmiennej SENSORS
			delay(100);  							//aktywnemu czujnikowi odpowiada stan logiczny "1" mimo,i
			SCK = 1;									//transoptor w przypadku zaczenia czujnika daje "0"
			delay(400); 
			if (SENSORS != 0)						//jeli zmienna ma warto <>0,to aktywna jest ktra czujka
			{											//nastpuje wyzerowanie watchdoga - w ten sposb zyskuj 2sekundy -
				delay(100);							//ponowny odczyt dla upewnienia si
				readsensors();
				if (SENSORS != 0)
				{
					DELAYTIME--;					//rwne "time2on x 1,5s."
					delay(1000);
					if (DELAYTIME == 0) alarmon();	//jeli przekroczono czas opnienia,to zaczenie alarmu
				}
			}
				else DELAYTIME = time2on;
		}
			else
		{
			SCK = 1;									//wyczenie diody LED
			readsensors();							//odczyt czujnikw,interesuje nas bit poczony z X2
			if (SENSORS == 0x08)					//jeli bit jest ustawiony, to ponowny odczyt dla
			{											//upewnienia si, ale po czasie 500ms dla eliminacji zakce
				delay(500);
				readsensors();
				if (SENSORS == 0x01) 			//jeli bit nadal ustawiony, to opnienie na "ucieczk" i zaczenie
				{										//trybu czuwania (INWATCHMODE=ACTIVE)
														//ptla opnienia o czasie trwania ok. time2on x 2sekundy
					for (RECVD = time2on; RECVD > 0; RECVD--)
					{
						SCK = 0;						//zaczenie diody LED
						delay(1000);				//opnienie 1 sekunda
						SCK = 1;						//wyczenie diody LED
						delay(1000);				//opnienie 1 sekunda
					}
					INWATCHMODE = ACTIVE;		//zaczenie trybu czuwania
				}
			}
		}
	}
}

 



